<?php

declare(strict_types = 1);

namespace Drupal\file_extractor;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\SubformStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Provides helper methods to validate settings form.
 *
 * @package Drupal\file_extractor
 */
class ExtractionSettingsFormHelper {

  use StringTranslationTrait;

  /**
   * The default extraction settings.
   */
  const DEFAULT_EXTRACTION_SETTINGS = [
    'excluded_extensions' => 'aif art avi bmp gif ico mov oga ogv png psd ra ram rgb flv',
    // Default the configuration to a sensible amount of text to extract and
    // cache in the database. 1 million characters should be enough for most
    // cases.
    'number_first_bytes' => '1 MB',
    'max_filesize' => '0',
    'exclude_private' => TRUE,
  ];

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    if ($form_state instanceof SubformStateInterface) {
      $form_state = $form_state->getCompleteFormState();
    }
    // Use temporary storage to get default values from parent form.
    $temporary_values = $form_state->getTemporary() + self::DEFAULT_EXTRACTION_SETTINGS;

    $form['excluded_extensions'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Excluded file extensions'),
      '#description' => $this->t('File extensions that are excluded from extraction. Separate extensions with a space and do not include the leading dot.<br />Example: "aif art avi bmp gif ico mov oga ogv png psd ra ram rgb flv"<br />Extensions are internally mapped to a MIME type, so it is not necessary to put variations that map to the same type (e.g. tif is sufficient for tif and tiff)'),
      '#default_value' => $temporary_values['excluded_extensions'],
      '#size' => 80,
      '#maxlength' => 255,
    ];
    $form['number_first_bytes'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Maximum size of the extracted string'),
      '#description' => $this->t('Enter a value like "1000", "10 KB", "10 MB" or "10 GB" in order to restrict the max size of the content after extraction.<br /> Enter "0" for no limit restriction.'),
      '#default_value' => $temporary_values['number_first_bytes'],
      '#size' => 5,
    ];
    $form['max_filesize'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Maximum file size'),
      '#description' => $this->t('Enter a value like "10 KB", "10 MB" or "10 GB" in order to restrict the max file size of files that should be extracted.<br /> Enter "0" for no limit restriction.'),
      '#default_value' => $temporary_values['max_filesize'],
      '#size' => 10,
    ];
    $form['exclude_private'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Exclude private files'),
      '#description' => $this->t('Check this checkbox if you want to exclude private files from being extracted.'),
      '#default_value' => $temporary_values['exclude_private'],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    // Can't use core size validation due to a bug.
    // See https://www.drupal.org/project/drupal/issues/2359675.
    // Validate 'number_first_bytes'.
    $number_first_bytes = trim($form_state->getValue('number_first_bytes'));
    $error = $this->validateSize($number_first_bytes);
    if ($error) {
      $form_state->setError($form['number_first_bytes'], $this->t('The size limit option must contain a valid value. You may either enter "0" (for no restriction) or a string like "10 KB", "10 MB" or "10 GB".'));
    }

    // Validate 'max_filesize'.
    $max_filesize = trim($form_state->getValue('max_filesize'));
    $error = $this->validateSize($max_filesize);
    if ($error) {
      $form_state->setError($form['max_filesize'], $this->t('The maximum file size option must contain a valid value. You may either enter "0" (for no restriction) or a string like "10 KB", "10 MB" or "10 GB".'));
    }
  }

  /**
   * Helper method to validate the size of files' format.
   *
   * @param string $bytes
   *   Number of bytes.
   *
   * @return bool
   *   TRUE if $bites is of form "N KB", "N MB" or "N GB" where N is integer.
   */
  protected function validateSize(string $bytes) : bool {
    $error = FALSE;
    if ($bytes != '0') {

      $size_info = explode(' ', $bytes);
      // The only case we can have count($size_info) == 1 is for '0' value.
      if (count($size_info) == 1) {
        $error = $size_info[0] != '0';
      }
      elseif (count($size_info) != 2) {
        $error = TRUE;
      }
      else {
        $starts_integer = is_int((int) $size_info[0]);
        $unit_expected = in_array($size_info[1], ['KB', 'MB', 'GB']);
        $error = !$starts_integer || !$unit_expected;
      }
    }
    return $error;
  }

}
