<?php

declare(strict_types = 1);

namespace Drupal\file_extractor\Plugin\file_extractor\Extractor;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\file\FileInterface;
use Drupal\file_extractor\Extractor\ExtractorPluginBase;

/**
 * Provides docconv extractor.
 *
 * @FileExtractorExtractor(
 *   id = "docconv_extractor",
 *   label = @Translation("Docconv Extractor"),
 *   description = @Translation("Adds Docconv extractor support."),
 * )
 */
class DocconvExtractor extends ExtractorPluginBase implements PluginFormInterface {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'docconv_path' => '/usr/bin/docd',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['docconv_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Docconv binary'),
      '#description' => $this->t('Enter the name of @binary executable or the full path to the @binary binary. Example: "@binary_example" or "@full_path_example".', [
        '@binary' => 'docconv',
        '@binary_example' => 'docd',
        '@full_path_example' => '/usr/bin/docd',
      ]),
      '#default_value' => $this->configuration['docconv_path'],
      '#required' => TRUE,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    $docconv_path = $form_state->getValue('docconv_path');
    if (!$this->isBinaryName($docconv_path) && !file_exists($docconv_path)) {
      $form_state->setError($form['docconv_path'], $this->t('The file %path does not exist.', ['%path' => $docconv_path]));
    }

    parent::validateConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function extract(FileInterface $file) : string {
    $log_variables = [];
    $docconv_path = $this->configuration['docconv_path'];
    $file_path = $this->getRealpath($file->getFileUri());

    if (!$this->checkBinaryAndFile($docconv_path, $file_path, $log_variables)) {
      return '';
    }

    $output = $this->execExtraction($docconv_path, [
      '-input',
      escapeshellcmd($file_path),
    ]);
    if (is_null($output)) {
      $this->logger->error('An error occurred during the extraction of the file @file_path with the binary @binary_path.', $log_variables);
      return '';
    }
    return $output;
  }

}
