<?php

declare(strict_types = 1);

namespace Drupal\file_extractor\Plugin\file_extractor\Extractor;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\file\FileInterface;
use Drupal\file_extractor\Extractor\ExtractorPluginBase;

/**
 * Provides tika extractor.
 *
 * @FileExtractorExtractor(
 *   id = "tika_extractor",
 *   label = @Translation("Tika Extractor"),
 *   description = @Translation("Adds Tika extractor support."),
 * )
 */
class TikaExtractor extends ExtractorPluginBase implements PluginFormInterface {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'java_path' => 'java',
      'tika_path' => '',
      'tika_config_path' => '',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['java_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Java binary'),
      '#description' => $this->t('Enter the name of @binary executable or the full path to the @binary binary. Example: "@binary_example" or "@full_path_example".', [
        '@binary' => 'java',
        '@binary_example' => 'java',
        '@full_path_example' => '/usr/bin/java',
      ]),
      '#default_value' => $this->configuration['java_path'],
      '#required' => TRUE,
    ];
    $form['tika_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Path to Tika .jar file'),
      '#description' => $this->t('Enter the full path to the Tika executable jar file. Example: "/var/apache-tika/tika-app-1.8.jar".'),
      '#default_value' => $this->configuration['tika_path'],
      '#required' => TRUE,
    ];
    $form['tika_config_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Path to Tika config file'),
      '#description' => $this->t('Enter the full path to the Tika config file. Example: "/var/apache-tika/tika-config.xml".'),
      '#default_value' => $this->configuration['tika_config_path'],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    $java_path = $form_state->getValue('java_path');
    $tika_path = $form_state->getValue('tika_path');
    $tika_config_path = $form_state->getValue('tika_config_path');
    $output = [];
    $return_code = NULL;

    // Check Java path.
    exec($java_path, $output, $return_code);
    // $return_code = 127 if it fails. 1 instead.
    if ($return_code != 1) {
      $form_state->setError($form['java_path'], $this->t('Invalid path or filename %path for Java binary.', ['%path' => $java_path]));
      return;
    }

    // Check Tika path.
    if (!file_exists($tika_path)) {
      $form_state->setError($form['tika_path'], $this->t('The file %path does not exist.', ['%path' => $tika_path]));
    }
    // Check return code.
    else {
      $cmd = $java_path . ' -jar ' . escapeshellarg($tika_path) . ' -V';
      exec($cmd, $output, $return_code);
      // $return_code = 1 if it fails. 0 instead.
      if ($return_code) {
        $form_state->setError($form['tika_path'], $this->t('Tika could not be reached and executed.'));
      }
    }

    // Check Tika config path.
    if (!empty($tika_config_path) && !file_exists($tika_config_path)) {
      $form_state->setError($form['tika_config_path'], $this->t('The file %path does not exist.', ['%path' => $tika_config_path]));
    }

    parent::validateConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function extract(FileInterface $file) : string {
    $log_variables = [];
    $java_path = $this->configuration['java_path'];
    $tika_path = realpath($this->configuration['tika_path']);
    $tika_config_path = '';
    if (!empty($this->configuration['tika_config_path'])) {
      $tika_config_path = realpath($this->configuration['tika_config_path']);
    }
    $file_path = $this->getRealpath($file->getFileUri());

    // Check Java binary and Tika file.
    if (!$this->checkBinaryAndFile($java_path, $tika_path, $log_variables)) {
      return '';
    }

    // Check file to extract. It will re-check Java binary; but this avoid
    // code duplication.
    if (!$this->checkBinaryAndFile($java_path, $file_path, $log_variables)) {
      return '';
    }

    $command_prefix = '';
    if (strpos(ini_get('extension_dir'), 'MAMP/')) {
      $command_prefix = 'export DYLD_LIBRARY_PATH=""; ';
    }

    $command_arguments = [];
    // Force running the Tika jar headless.
    $command_arguments[] = '-Djava.awt.headless=true';
    if ($file->getMimeType() != 'audio/mpeg') {
      $command_arguments[] = '-Dfile.encoding=UTF8 -cp';
      $command_arguments[] = escapeshellarg($tika_path);
    }
    $command_arguments[] = '-jar';
    $command_arguments[] = escapeshellarg($tika_path);
    if (!empty($tika_config_path)) {
      $command_arguments[] = '--config=' . escapeshellarg($tika_config_path);
    }
    $command_arguments[] = '-t';
    $command_arguments[] = escapeshellarg($file_path);

    $output = $this->execExtraction($java_path, $command_arguments, $command_prefix);
    if (is_null($output)) {
      $this->logger->error('An error occurred during the extraction of the file @file_path with the binary @binary_path.', $log_variables);
      return '';
    }
    return $output;
  }

}
