<?php

declare(strict_types = 1);

namespace Drupal\file_extractor\Service;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Component\Utility\Bytes;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\KeyValueStore\KeyValueFactoryInterface;
use Drupal\file\FileInterface;
use Drupal\file_extractor\Event\FileIndexableEvent;
use Drupal\file_extractor\ExtractionSettingsFormHelper;
use Drupal\file_extractor\Extractor\ExtractorPluginManager;
use Drupal\file_extractor\Form\SettingsForm;
use Drupal\file_extractor\HookHandler\CacheHandler;
use Psr\Log\LoggerInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Symfony\Component\HttpFoundation\File\MimeType\MimeTypeGuesserInterface;

/**
 * Manager which provides methods to extract files.
 *
 * @package Drupal\file_extractor\Service
 */
class ExtractorManager implements ExtractorManagerInterface {

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The extractor plugin manager.
   *
   * @var \Drupal\file_extractor\Extractor\ExtractorPluginManager
   */
  protected $extractorPluginManager;

  /**
   * Logger.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * The mime type guesser.
   *
   * @var \Symfony\Component\HttpFoundation\File\MimeType\MimeTypeGuesserInterface
   */
  protected $mimeTypeGuesser;

  /**
   * The event dispatcher service.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $eventDispatcher;

  /**
   * Key value service.
   *
   * @var \Drupal\Core\KeyValueStore\KeyValueFactoryInterface
   */
  protected $keyValue;

  /**
   * Extraction settings.
   *
   * @var array
   */
  protected $extractionSettings = [];

  /**
   * Store the calculated mime types for a list of extensions string.
   *
   * @var string[][]
   */
  protected $mimesForExtensions = [];

  /**
   * ExtractorManager constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\file_extractor\Extractor\ExtractorPluginManager $extractor_plugin_manager
   *   The extractor plugin manager.
   * @param \Psr\Log\LoggerInterface $logger
   *   Logger.
   * @param \Symfony\Component\HttpFoundation\File\MimeType\MimeTypeGuesserInterface $mimeTypeGuesser
   *   Mime type guesser service.
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $event_dispatcher
   *   The event dispatcher service.
   * @param \Drupal\Core\KeyValueStore\KeyValueFactoryInterface $key_value
   *   Key value service.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    ExtractorPluginManager $extractor_plugin_manager,
    LoggerInterface $logger,
    MimeTypeGuesserInterface $mimeTypeGuesser,
    EventDispatcherInterface $event_dispatcher,
    KeyValueFactoryInterface $key_value
  ) {
    $this->configFactory = $config_factory;
    $this->extractorPluginManager = $extractor_plugin_manager;
    $this->logger = $logger;
    $this->mimeTypeGuesser = $mimeTypeGuesser;
    $this->eventDispatcher = $event_dispatcher;
    $this->keyValue = $key_value;

    // Set extraction settings default value. Try to load from global config,
    // complete with default values from ExtractionSettingsFormHelper.
    $config = $this->configFactory->get(SettingsForm::CONFIG_NAME);
    $config_extraction_settings = is_null($config->get('extraction_settings')) ? [] : $config->get('extraction_settings');
    $this->extractionSettings = $config_extraction_settings + ExtractionSettingsFormHelper::DEFAULT_EXTRACTION_SETTINGS;
  }

  /**
   * {@inheritdoc}
   */
  public function extract(FileInterface $file) : string {
    if (!$this->isFileIndexable($file)) {
      return '';
    }

    $config = $this->configFactory->get(SettingsForm::CONFIG_NAME);
    // Get cache or extract.
    // This can lead to a wrong behavior because extraction settings can be
    // overridden and the cache key is the same. Currently there is only the
    // 'number_first_bytes' settings that can impact cache data and it is
    // unlikely that this settings differs across a website, so it limits the
    // probability of bug.
    // It is possible to put a config hash in the cache key, but this would
    // prevent the possibility to flush cache on 'file update' and 'file delete'
    // hooks. And impossible to query the database because it would lead to
    // errors if the key_value is managed by another backend.
    $key = $file->id();
    if ($config->get('cache_enable')) {
      $cache = $this->keyValue->get(CacheHandler::KEY_VALUE_COLLECTION)->get($key);
      if ($cache) {
        $extracted_data = $cache;
      }
    }
    else {
      $extracted_data = $this->extractFile($file);
      if ($config->get('cache_enable')) {
        $this->keyValue->get(CacheHandler::KEY_VALUE_COLLECTION)->set($key, $extracted_data);
      }
    }

    return $extracted_data;
  }

  /**
   * {@inheritdoc}
   */
  public function setExtractionSettings(array $extraction_settings): void {
    $this->extractionSettings = $extraction_settings + $this->extractionSettings;
  }

  /**
   * Helper function.
   *
   * @param \Drupal\file\FileInterface $file
   *   The file entity to extract.
   *
   * @return string
   *   The file extracted content.
   */
  protected function extractFile(FileInterface $file) : string {
    $config = $this->configFactory->get(SettingsForm::CONFIG_NAME);
    $extractor_plugin_id = $config->get('extraction_method');
    $extractor_plugin_settings = $config->get('extraction_method_settings');

    if (is_null($extractor_plugin_id)) {
      $this->logger->error('No extraction plugin had been configured yet.');
      return '';
    }

    // Do not return if extractor plugin settings is null because a plugin may
    // not have configuration.
    if (is_null($extractor_plugin_settings)) {
      $extractor_plugin_settings = [];
    }

    try {
      $extractor_plugin = $this->extractorPluginManager->createInstance($extractor_plugin_id, $extractor_plugin_settings);
    }
    catch (PluginException $exception) {
      $this->logger->error('An error occurred when trying to instantiate an Extractor plugin with ID: @plugin_id. Error message was @msg', [
        '@plugin_id' => $extractor_plugin_id,
        '@msg' => $exception->getMessage(),
      ]);
      return '';
    }

    return $this->limitBytes($extractor_plugin->extract($file));
  }

  /**
   * Check if the file is allowed to be indexed.
   *
   * @param \Drupal\file\FileInterface $file
   *   A file object.
   *
   * @return bool
   *   TRUE or FALSE
   */
  protected function isFileIndexable(FileInterface $file) : bool {
    // File should exist.
    if (!file_exists($file->getFileUri())) {
      return FALSE;
    }

    // File should have a mime type that is allowed.
    $all_excluded_mimes = $this->getMimesForExtensions($this->extractionSettings['excluded_extensions']);
    if (in_array($file->getMimeType(), $all_excluded_mimes)) {
      return FALSE;
    }

    // File permanent.
    if (!$file->isPermanent()) {
      return FALSE;
    }

    // File shouldn't exceed configured file size.
    if (!$this->isFileSizeAllowed($file)) {
      return FALSE;
    }

    // Whether a private file can be indexed or not.
    if (!$this->isPrivateFileAllowed($file)) {
      return FALSE;
    }

    // Allow other modules to decide if the file is indexable with an event.
    $event = new FileIndexableEvent($file);
    $this->eventDispatcher->dispatch(FileIndexableEvent::EVENT_NAME, $event);
    if (!$event->isIndexable()) {
      return FALSE;
    }

    return TRUE;
  }

  /**
   * Get a corresponding array of mime types.
   *
   * Obtained from a space separated string of file extensions.
   *
   * @param string $extensions_string
   *   Extensions string.
   *
   * @return array
   *   Array or mimes.
   */
  protected function getMimesForExtensions(string $extensions_string) : array {
    if (!isset($this->mimesForExtensions[$extensions_string])) {
      $extensions = explode(' ', $extensions_string);
      $mimes = [];
      foreach ($extensions as $extension) {
        $mimes[] = $this->mimeTypeGuesser->guess('dummy.' . $extension);
      }
      // Ensure we get an array of unique mime values because many extension can
      // map the the same mime type.
      $this->mimesForExtensions[$extensions_string] = array_unique($mimes);
    }

    return $this->mimesForExtensions[$extensions_string];
  }

  /**
   * Exclude files that exceed configured max size.
   *
   * @param \Drupal\file\FileInterface $file
   *   File object.
   *
   * @return bool
   *   TRUE if the file size does not exceed configured max size.
   */
  protected function isFileSizeAllowed(FileInterface $file) : bool {
    $max_file_size = $this->extractionSettings['max_filesize'];
    if (!empty($max_file_size) && $max_file_size != '0') {
      $file_size_bytes = $file->getSize();
      $max_size_bytes = Bytes::toInt($max_file_size);
      if ($file_size_bytes > $max_size_bytes) {
        return FALSE;
      }
    }

    return TRUE;
  }

  /**
   * Exclude private files from being indexed.
   *
   * @param \Drupal\file\FileInterface $file
   *   File object.
   *
   * @return bool
   *   TRUE if we should prevent current file from being indexed.
   */
  protected function isPrivateFileAllowed(FileInterface $file) : bool {
    $exclude_private = $this->extractionSettings['exclude_private'];

    // Know if private files are allowed to be indexed.
    $private_allowed = !$exclude_private;
    // Know if current file is private.
    $uri = $file->getFileUri();
    $file_is_private = FALSE;
    if (substr($uri, 0, 10) == 'private://') {
      $file_is_private = TRUE;
    }

    if (!$file_is_private) {
      return TRUE;
    }
    else {
      return $private_allowed;
    }
  }

  /**
   * Limit the indexed text to first N bytes.
   *
   * @param string $extracted_text
   *   The whole extracted text.
   *
   * @return string
   *   The first N bytes of the extracted text that will be indexed and cached.
   */
  protected function limitBytes(string $extracted_text) : string {
    // Bytes::toInt should be an int, but in practice it is float.
    $bytes = intval(Bytes::toInt($this->extractionSettings['number_first_bytes']));

    // If $bytes is 0 return all items.
    if ($bytes == 0) {
      return $extracted_text;
    }
    else {
      $extracted_text = mb_strcut($extracted_text, 0, $bytes);
    }
    return $extracted_text;
  }

}
